/*
 *  Copyright 2007 Tacos.
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 * 
 *       http://www.apache.org/licenses/LICENSE-2.0
 * 
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *  under the License.
 */
package net.sf.tacos.asset;

import java.io.InputStream;

import org.apache.tapestry.IAsset;
import org.apache.hivemind.Resource;
import org.apache.hivemind.Location;
import org.apache.tapestry.services.AbsoluteURLBuilder;

/**
 * Decorates an asset in order to generate an absolute url.
 * @author Andreas Andreou
 */
public class AbsoluteAsset implements IAsset {

    private IAsset asset;
    private AbsoluteURLBuilder absoluteURLBuilder;

    public AbsoluteAsset(IAsset asset, AbsoluteURLBuilder absoluteURLBuilder) {
        this.asset = asset;
        this.absoluteURLBuilder = absoluteURLBuilder;
    }

    public String buildURL() {
        return absoluteURLBuilder.constructURL(asset.buildURL());
    }

    public InputStream getResourceAsStream() {
        return asset.getResourceAsStream();
    }

    public Resource getResourceLocation() {
        return asset.getResourceLocation();
    }

    public Location getLocation() {
        return asset.getLocation();
    }
}

