/*
 *  Copyright 2007 Tacos.
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 * 
 *       http://www.apache.org/licenses/LICENSE-2.0
 * 
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *  under the License.
 */
package net.sf.tacos.binding;

import net.sf.tacos.asset.AbsoluteAsset;
import org.apache.hivemind.Location;
import org.apache.hivemind.util.Defense;
import org.apache.tapestry.BindingException;
import org.apache.tapestry.IAsset;
import org.apache.tapestry.IComponent;
import org.apache.tapestry.binding.AbstractBinding;
import org.apache.tapestry.coerce.ValueConverter;
import org.apache.tapestry.services.AbsoluteURLBuilder;

/**
 * A binding that is connected to an asset provided by a component.
 * 
 * @author Andreas Andreou
 * @since 4.1
 */
public class AbsoluteAssetBinding extends AbstractBinding
{
    private final IComponent component;
    private AbsoluteURLBuilder absoluteURLBuilder;

    public AbsoluteAssetBinding(String description, ValueConverter valueConverter, 
            AbsoluteURLBuilder absoluteURLBuilder, Location location, IComponent component, String assetName)
    {
        super(assetName, valueConverter, location);

        Defense.notNull(component, "component");
        Defense.notNull(assetName, "assetName");

        this.component = component;
        this.absoluteURLBuilder = absoluteURLBuilder;
    }

    public Object getObject()
    {
        IAsset asset = component.getAsset(_description);

        if (asset == null)
            throw new BindingException("Component " + component.getExtendedId() + 
                    " does not contain an asset named ''" + _description + "''.",
                    component, getLocation(), this, null);

        return new AbsoluteAsset(asset, absoluteURLBuilder);
    }

    public Object getComponent()
    {
        return component;
    }
}

