package net.sf.tacos.components;

import java.util.HashMap;
import java.util.Map;

import org.apache.tapestry.BaseComponent;
import org.apache.tapestry.IMarkupWriter;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.IScript;
import org.apache.tapestry.TapestryUtils;
import org.apache.tapestry.form.IFormComponent;

/**
 * Provides a hint for a form field to guide the user on how to fill it.
 * The box with the helper text appears when the field is focused and disappears when the focus is lost.
 * 
 * @author Igor Drobiazko
 */
public abstract class FieldHint extends BaseComponent {
	/**
     * {@inheritDoc}
     */
	protected void renderComponent(IMarkupWriter writer, IRequestCycle cycle) {
		writer.begin("span");
        renderIdAttribute(writer, cycle);
        writer.appendAttribute("class", "hint");
        renderBody(writer, cycle);
        super.renderComponent(writer, cycle);
        writer.end();
        
        if (!cycle.isRewinding()) {
            Map parms = new HashMap();
            parms.put("component", this);
            parms.put("startEvent", getStartEvent());
            parms.put("endEvent", getEndEvent());

            getScript().execute(this, cycle, TapestryUtils.getPageRenderSupport(cycle, this), parms);
            }
	}
	
    /** Injected {@link IScript}. */
    public abstract IScript getScript();
    
    /** Field parameter. */
    public abstract IFormComponent getField();
    
    /** The js event of the form component that should make the hint visible. */
    public abstract String getStartEvent();
    
    /** The js event of the form component that should make the hint hidden. */
    public abstract String getEndEvent();    
}
