package net.sf.tacos.components.dojo;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.apache.tapestry.IActionListener;
import org.apache.tapestry.IDirect;
import org.apache.tapestry.IMarkupWriter;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.IScript;
import org.apache.tapestry.Tapestry;
import org.apache.tapestry.TapestryUtils;
import org.apache.tapestry.dojo.AbstractWidget;
import org.apache.tapestry.engine.DirectServiceParameter;
import org.apache.tapestry.engine.IEngineService;
import org.apache.tapestry.engine.ILink;
import org.apache.tapestry.listener.ListenerInvoker;

public abstract class DropTarget extends AbstractWidget implements IDirect{
	/**
	 * Name of the {@link IRequestCycle} parameter whose value is the
	 * identifier of the dropped draggable source
	 * @see {@link IRequestCycle#getParameter(String)}
	 */
	public static final String DRAG_SOURCE_PARAMETER = "dragSourceId";
	
	/**
	 * {@inheritDoc}
	 */
	public void renderWidget(IMarkupWriter writer, IRequestCycle cycle) {
        writer.begin(getTemplateTagName());
        renderIdAttribute(writer, cycle);
        renderInformalParameters(writer, cycle);
        renderBody(writer, cycle);
        writer.end();
        
        if (!cycle.isRewinding()) {
            Map params = new HashMap();
            params.put("component", this);
            params.put("dragSource", getDragSource());
            params.put("dragSourceElement", getDragSourceElement());
            params.put("dragSourceTag", getDragSourceTag());
            params.put("acceptedTypes", getAcceptedTypes());
            params.put("insertDragSource", new Boolean(isInsertDragSource()));
            params.put("onDrop", getOnDrop());
            if (getListener()!=null)
                params.put("url", getLink().getURL());        

            getScript().execute(this, cycle, TapestryUtils.getPageRenderSupport(cycle, this), params);
        }
	}
	
	/**
	 * {@inheritDoc}
	 */
	public void trigger(IRequestCycle cycle) {
        IActionListener listener = getListener();
        if (listener == null)
            throw Tapestry.createRequiredParameterException(this, "listener");
        
        getListenerInvoker().invokeListener(listener, this, cycle);
	}
	
	/**Returns a link*/
	private ILink getLink(){
		DirectServiceParameter dsp = new DirectServiceParameter(this);
		return getEngine().getLink(isStateful(), dsp);
	}
	
    /** Reference to js function - called when a drop event is generated.*/
    public abstract String getOnDrop();        
    /** List item source parameter */
    public abstract Object getDragSource();
    /** Drag source element */
    public abstract String getDragSourceElement();
    /** Drag source tag */
    public abstract String getDragSourceTag();
    /** Type of tags target accepts */
    public abstract Collection getAcceptedTypes();
    /** Whether or not to allow insertion of source element */
    public abstract boolean isInsertDragSource();
    /** Injected listener parameter, may be null. */
    public abstract IActionListener getListener();
    /** Injected script. */
    public abstract IScript getScript();
    /** Injected listener invoker. */
    public abstract ListenerInvoker getListenerInvoker();
    /** Injected engine service . */
    public abstract IEngineService getEngine();

}
