// Copyright May 4, 2006 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
package net.sf.tacos.components.dojo.form;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.apache.tapestry.IMarkupWriter;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.IScript;
import org.apache.tapestry.PageRenderSupport;
import org.apache.tapestry.TapestryUtils;
import org.apache.tapestry.dojo.form.AbstractFormWidget;
import org.apache.tapestry.engine.IEngineService;
import org.apache.tapestry.form.TranslatedField;
import org.apache.tapestry.form.TranslatedFieldSupport;
import org.apache.tapestry.form.ValidatableFieldSupport;
import org.apache.tapestry.json.JSONObject;
import org.apache.tapestry.services.DataSqueezer;
import org.apache.tapestry.valid.ValidatorException;

/**
 * An html field similar to a <code>textarea</code> field that 
 * is wrapped by a dojo Editor2 widget.<p/>
 * This component adds javascript that makes the dojo widget
 * update the value of the textarea (to support async submits) it is bound to. 
 * 
 * @author andyhot
 */
public abstract class HtmlArea extends AbstractFormWidget implements TranslatedField
{    
    /**
     * 
     * {@inheritDoc}
     */
    protected void renderFormWidget(IMarkupWriter writer, IRequestCycle cycle)
    {
        String value = getTranslatedFieldSupport().format(this, getValue());
        
        renderDelegatePrefix(writer, cycle);
        
        writer.begin("textarea");
        writer.attribute("name", getName());
        
        if (isDisabled())
            writer.attribute("disabled", "disabled");
        
        renderIdAttribute(writer, cycle);
        
        renderDelegateAttributes(writer, cycle);
        
        getTranslatedFieldSupport().renderContributions(this, writer, cycle);
        getValidatableFieldSupport().renderContributions(this, writer, cycle);
        
        renderInformalParameters(writer, cycle);
        
        if (value != null) writer.print(value);

        writer.end();

        renderDelegateSuffix(writer, cycle);
              
        Map parms = new HashMap();
        parms.put("id", getClientId());
        
        JSONObject json = new JSONObject();        
        json.put((Object)"widgetId", getName());
        json.put((Object)"name", getName());
        json.put((Object)"disabled", isDisabled());
        if (getToolbarTemplatePath()!=null)
            json.put((Object)"toolbarTemplatePath", getToolbarTemplatePath());
        
        parms.put("props", json.toString());
        parms.put("widget", this);
        
        PageRenderSupport prs = TapestryUtils.getPageRenderSupport(cycle, this);
        getScript().execute(this, cycle, prs, parms);
    }
    
    /**
     * @see org.apache.tapestry.form.AbstractFormComponent#rewindFormComponent(org.apache.tapestry.IMarkupWriter, org.apache.tapestry.IRequestCycle)
     */
    protected void rewindFormWidget(IMarkupWriter writer, IRequestCycle cycle)
    {
        String value = cycle.getParameter(getName());
        
        try
        {
            String text = (String) getTranslatedFieldSupport().parse(this, value);
            
            getValidatableFieldSupport().validate(this, writer, cycle, text);
            
            setValue(text);
        }
        catch (ValidatorException e)
        {
            getForm().getDelegate().record(e);
        }
    }    
    
    public abstract String getToolbarTemplatePath();
        
    /** @since 2.2 * */
    public abstract Object getValue();

    /** @since 2.2 * */
    public abstract void setValue(Object value);
    
    /** Injected. */
    public abstract DataSqueezer getDataSqueezer();
    
    /**
     * Injected.
     */
    public abstract ValidatableFieldSupport getValidatableFieldSupport();
    
    /**
     * Injected.
     */
    public abstract TranslatedFieldSupport getTranslatedFieldSupport();    

    /**
     * Injected.
     */
    public abstract IEngineService getDirectService();
    
    /**
     * Injected.
     */
    public abstract IScript getScript();
    
    /**
     * @see org.apache.tapestry.form.AbstractFormComponent#isRequired()
     */
    public boolean isRequired()
    {
        return getValidatableFieldSupport().isRequired(this);
    }

    /** 
     * {@inheritDoc}
     */
    public List getUpdateComponents()
    {
        List comps = new ArrayList();
        comps.add(getClientId());
        
        return comps;
    }
    
    /** 
     * {@inheritDoc}
     */
    public boolean isAsync()
    {
        return false;
    }
    
    /** 
     * {@inheritDoc}
     */
    public boolean isJson()
    {
        return false;
    }
}
