package net.sf.tacos.components.scriptaculous;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.tapestry.AbstractComponent;
import org.apache.tapestry.IMarkupWriter;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.IScript;
import org.apache.tapestry.TapestryUtils;
import org.apache.tapestry.json.JSONObject;

/**
 * Implementation of the <a href="http://wiki.script.aculo.us/scriptaculous/show/Draggables">Draggables</a>.
 * <p>
 * This component makes html elements draggable. Draggables can be dragged and dropped into Droppables.
 * </p>
 * 
 * <p>Example:
 * <pre>&lt;div jwcid="@tacos:Draggable" draggables="ognl:{'id1', 'id2', 'id3'}" /&gt;</pre>
 * </p>
 * @author Igor Drobiazko
 * @since 4.1
 * @see Droppable
 */
public abstract class Draggable extends AbstractComponent {
	
	/**
	 * {@inheritDoc}
	 */
	protected void renderComponent(IMarkupWriter writer, IRequestCycle cycle) {
        renderBody(writer, cycle);
        
        JSONObject json = new JSONObject();
        json.put("revert", new Boolean(isRevert()));
        json.put("ghosting", new Boolean(isGhosting()));
        
        Map parms = new HashMap();
        parms.put("props", json.toString());
        parms.put("draggables", getDraggables());
        
        getScript().execute(this, cycle, TapestryUtils.getPageRenderSupport(cycle, this), parms);
	}
	
	/**
	 * Returns {@link String[]} or {@link Collection} of element ids to make draggable.
	 */
    public abstract List getDraggables();
    
    /**
     * If set to <code>true</code>, clones the element and drags the clone,
     * leaving the original in place until the clone is dropped.
     */
	public abstract boolean isGhosting();
	
	/**
	 * If set to <code>true</code>, the element returns to its original position when the drags ends.
	 */
	public abstract boolean isRevert();
	
    /** Injected {@link IScript}. */
    public abstract IScript getScript();
}
