package net.sf.tacos.components.scriptaculous;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.apache.tapestry.AbstractComponent;
import org.apache.tapestry.IActionListener;
import org.apache.tapestry.IDirect;
import org.apache.tapestry.IMarkupWriter;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.IScript;
import org.apache.tapestry.Tapestry;
import org.apache.tapestry.TapestryUtils;
import org.apache.tapestry.engine.DirectServiceParameter;
import org.apache.tapestry.engine.IEngineService;
import org.apache.tapestry.engine.ILink;
import org.apache.tapestry.listener.ListenerInvoker;

/**
 * Implementation of the <a href="http://wiki.script.aculo.us/scriptaculous/show/Droppables">Droppables</a>.
 * <p>
 * This component makes a html element droppable. Droppables are elements which allow Draggables to be dropped on.
 * When a draggable is released over a droppable, a listener method may be triggered to inform your application
 * about the dropped element. 
 * </p>
 * 
 * <p>Example:
 * <pre>&lt;div jwcid="@tacos:Droppable" listener="listener:onDrop" hoverClass="literal:hoverCss" /&gt;</pre>
 * </p>
 * 
 * @author Igor Drobiazko
 * @since 4.1
 * @see Draggable
 */
public abstract class Droppable extends AbstractComponent implements IDirect{
	/**
	 * Name of the {@link IRequestCycle} parameter whose value is the
	 * identifier of the dropped draggable
	 * @see {@link IRequestCycle#getParameter(String)}
	 */
	public static final String DRAGGABLE_PARAMETER = "draggable";
	
	/**
	 * {@inheritDoc}
	 */
	protected void renderComponent(IMarkupWriter writer, IRequestCycle cycle) {
        writer.begin(getTemplateTagName());
        renderIdAttribute(writer, cycle);
        renderInformalParameters(writer, cycle);
        renderBody(writer, cycle);
        writer.end();
        
        Map params = new HashMap();
        params.put("component", this);
        params.put("accept", getAccept());
        params.put("hoverClass", getHoverClass());
        if(getListener()!=null)
        	params.put("url", getLink().getURL());
        
        getScript().execute(this, cycle, TapestryUtils.getPageRenderSupport(cycle, this), params);
	}
	
	/**
	 * {@inheritDoc}
	 */
	public void trigger(IRequestCycle cycle) {
        IActionListener listener = getListener();
        if (listener == null)
            throw Tapestry.createRequiredParameterException(this, "listener");
        
        getListenerInvoker().invokeListener(listener, this, cycle);
	}
	
	/**Returns a link*/
	private ILink getLink(){
		return getEngine().getLink(isStateful(), new DirectServiceParameter(this));
	}

    /** Injected listener parameter, may be null. */
    public abstract IActionListener getListener();
    /** Injected script. */
    public abstract IScript getScript();
    /** Injected listener invoker. */
    public abstract ListenerInvoker getListenerInvoker();
    /** Injected engine service . */
    public abstract IEngineService getEngine();
	/**
	 * Returns {@link String[]} or {@link Collection} of strings describing CSS classes.
	 * The Droppable will only accept {@link Draggable} that have one or more of these CSS classes.
	 */
    public abstract Collection getAccept();
    /**
     * Returns the CSS class this Droppable will have
     * when an accepted {@link Draggable} is hovered over it.
     */
    public abstract String getHoverClass();
}
