/*
 * TreeHelper.java
 *
 * Created on February 4, 2007, 11:54 PM
 */

package net.sf.tacos.components.tree;

import java.util.Stack;
import org.apache.tapestry.AbstractComponent;
import org.apache.tapestry.IMarkupWriter;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.engine.NullWriter;

/**
 *
 * @author andyhot
 */
public abstract class TreeHelper extends AbstractComponent {
    
    private Stack stack;
    
    public abstract TreeIterator getTreeIterator();
    
    private Stack getCycleStack(IRequestCycle cycle) {
        Stack currStack = (Stack) cycle.getAttribute(TreeHelper.class.getName());
        if (currStack == null) {
            currStack = new Stack();
        }
        return currStack;
    }
    
    private void setCycleStack(IRequestCycle cycle, Stack currStack) {
        cycle.setAttribute(TreeHelper.class.getName(), currStack);
    }

    protected void renderComponent(IMarkupWriter writer, IRequestCycle cycle) {          
        stack = getCycleStack(cycle);
        if (getTreeIterator() == null) {
            finalRenderComponent();
        } else {  
            int diff = getTreeIterator().getDepth() - getTreeIterator().getPreviousDepth();
            boolean empty = stack.empty();
            
            if (writer == NullWriter.getSharedInstance()  && cycle.getResponseBuilder().isDynamic()
                    && (empty || stack.peek()==writer)) {
                renderBody(writer, cycle);
                return;
            }

            if (empty) {
                stack.push(writer.getNestedWriter()); 
                diff = 0;
            }
            IMarkupWriter writerGive = (IMarkupWriter) stack.peek();
            normalRenderComponent(writerGive, cycle, diff, empty);
        }
        
        setCycleStack(cycle, stack);
    }
    
    protected void finalRenderComponent() {
        while (!stack.empty()) {
            IMarkupWriter inner = (IMarkupWriter) stack.pop();            
            inner.close();
        }
        stack = null;        
        return;
    }
    
    protected void normalRenderComponent(IMarkupWriter writer, IRequestCycle cycle, 
            int diff, boolean empty) {                    
        //System.out.println("Doing: " + getClientId() + " Diff:" + diff + "Wr:" + writer);
        if (diff==0) {
            if (!empty)
                writer.end();
            
            writer.begin("div");            
            renderIdAttribute(writer, cycle);
            writer.closeTag();
            renderBody(writer, cycle);
        } else if (diff>0) {            
            writer = writer.getNestedWriter();
            stack.push(writer);
            
            writer.begin("div");            
            renderIdAttribute(writer, cycle);
            writer.closeTag();            
            renderBody(writer, cycle);            
        } else {
            writer = (IMarkupWriter) stack.pop();
            for (int c = diff; c < 0 && !stack.empty(); c++) {
                writer.close();
                writer = (IMarkupWriter) stack.pop();
            }
            if (stack.empty() && cycle.getResponseBuilder().isDynamic()) {
                renderBody(NullWriter.getSharedInstance(), cycle);
                writer.close();
                stack.push(NullWriter.getSharedInstance());
                return;
            }
            
            writer.end();
            stack.push(writer);
            
            writer.begin("div");            
            renderIdAttribute(writer, cycle);
            writer.closeTag();
            renderBody(writer, cycle);
        }
        
    }

}
