package net.sf.tacos.components.tree;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Stack;

/**
 * @author phraktle
 */
public abstract class TreeIterator implements Iterator {

    private final Stack todo = new Stack();
    private int depth = 0;
    private int previousDepth = 0;
    private int markers = 0;

    /**
     * New tree
     * @param rootElement
     */
    public TreeIterator(Object rootElement)
    {
        todo.push(rootElement);
    }

    /**
     * Tree iterator
     * @param rootElements
     */
    public TreeIterator(Collection rootElements)
    {
        push(rootElements);
    }

    /**
     * Depth
     * @return
     */
    public int getDepth()
    {
        return depth;
    }

    /**
     * Previous depth
     * @return
     */
    public int getPreviousDepth()
    {
        return previousDepth;
    }

    /**
     * 
     * {@inheritDoc}
     */
    public boolean hasNext()
    {
        return !todo.isEmpty() && todo.size() != markers;
    }

    /**
     * 
     * {@inheritDoc}
     */
    public Object next()
    {
        Object node;
        do {
            node = todo.pop();
            if (node == MARKER) {
                markers--;
            }
        } while(node == MARKER);
        previousDepth = depth;
        depth = markers;
        Collection children = getChildren(node);
        if (!children.isEmpty()) {
            todo.push(MARKER);
            markers++;
            push(children);
        }
        return node;
    }

    /**
     * Push set of elements
     * @param elements
     */
    private void push(Collection elements)
    {
        List rev = new ArrayList(elements);
        Collections.reverse(rev);
        todo.addAll(rev);
    }

    /**
     * Gets children
     * @param node
     * @return
     */
    protected abstract Collection getChildren(Object node);

    /**
     * 
     * {@inheritDoc}
     */
    public void remove()
    {
        throw new UnsupportedOperationException();
    }

    private final static Object MARKER = new Object();

}