package net.sf.tacos.components.tree;

import java.io.Serializable;
import java.util.Collection;
import java.util.Iterator;
import java.util.Set;
import net.sf.tacos.model.IKeyProvider;
import net.sf.tacos.model.ITreeContentProvider;
import org.apache.hivemind.util.Defense;

/**
 * @author phraktle
 */
public class TreeManager implements ITreeManager {
    
    private final Set state;
    private final ITreeContentProvider contentProvider;
    private final IKeyProvider keyProvider;
    
    /**
     * Creates tree manager.
     * @param state
     * @param contentProvider
     * @param keyProvider
     */
    public TreeManager(Set state, ITreeContentProvider contentProvider,
            IKeyProvider keyProvider)
    {
        Defense.notNull(state, "tree state");
        Defense.notNull(contentProvider, "tree contentProvider");
        Defense.notNull(keyProvider, "tree keyProvider");
        
        this.state = state;
        this.contentProvider = contentProvider;
        this.keyProvider = keyProvider;
    }

    /**
     * 
     * {@inheritDoc}
     */
    public boolean isExpanded(Object element)
    {
        return isExpandedKey(keyProvider.getKey(element));
    }

    /**
     * 
     * {@inheritDoc}
     */
    public boolean isExpandedKey(Serializable key)
    {
        return state.contains(key);
    }

    /**
     * 
     * {@inheritDoc}
     */
    public void setExpanded(Object element, boolean open)
    {
        setExpandedKey(keyProvider.getKey(element), open);
    }

    /**
     * 
     * {@inheritDoc}
     */
    public void setExpandedKey(Serializable key, boolean open)
    {
        if (open) {
            state.add(key);
        } else {
            state.remove(key);
        }
    }

    /**
     * 
     * @return
     */
    public Set getState()
    {
        return state;
    }

    /**
     * 
     * {@inheritDoc}
     */
    public void collapseAll()
    {
        state.clear();
    }
    
    /**
     * 
     * {@inheritDoc}
     */
    public Object reveal(Object element)
    {
        Object parent = contentProvider.getParent(element);
        if (parent != null) {
            setExpanded(parent, true);
            return reveal(parent);
        } else return element;
    }

    /**
     * 
     * {@inheritDoc}
     */
    public void expandAll()
    {
        expand(contentProvider.getElements());
    }

    /**
     * Expands all elements
     * @param rootElements
     */
    private void expand(Collection rootElements)
    {
        for(Iterator i = rootElements.iterator(); i.hasNext();) {
            Object element = i.next();
            setExpanded(element, true);
            if (contentProvider.hasChildren(element)) {
                expand(contentProvider.getChildren(element));
            }
        }
    }

}