/*
 * BeanTreeContent.java
 *
 * Created on February 4, 2007, 12:54 PM
 */

package net.sf.tacos.model.impl;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import net.sf.tacos.model.ITreeContentProvider;
import org.apache.commons.beanutils.PropertyUtils;

/**
 * A useful {@link ITreeContentProvider} that can recreate the tree structure
 * using a root bean (of any type) and the property name that will return its
 * children (a {@link Collection} of the same type).
 * 
 * This class is meant to be used for quickly prototyping trees. You're advised to
 * create your own implementation in order to tweek performance issues.
 * 
 * @author andyhot
 */
public class BeanWithChildrenTreeContentProvider implements ITreeContentProvider {
    
    private Object root;
    private String childProperty;
    private Map parents;
    
    public BeanWithChildrenTreeContentProvider(Object root, String childProperty) {
        this.root = root;
        this.childProperty = childProperty;
        buildParents();
    }
    
    private void buildParents() {
        parents = new HashMap();
        appendChildren(root, parents);
    }
    
    private void appendChildren(Object base, Map map) {
        for (Iterator iter = getChildren(base).iterator(); iter.hasNext();) {
            Object object = iter.next();
            map.put(object, base);
            appendChildren(object, map);
        }
    }    
    
    public Collection getChildren(Object parentElement) {
        try {
            Collection children = (Collection) PropertyUtils.getProperty(parentElement, childProperty);
            if (children == null)
                children = Collections.emptyList();
            return children;
        } catch (Exception e) {
            throw new RuntimeException("Error getting property", e);
        }           
    }

    public boolean hasChildren(Object parentElement) {
        return getChildren(parentElement).size() > 0;
    }

    public Object getParent(Object childElement) {
        return parents.get(childElement);        
    }

    public List getElements() {        
        return Collections.singletonList(root);
    }

}
