/*
 *  Copyright 2007 Tacos.
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 * 
 *       http://www.apache.org/licenses/LICENSE-2.0
 * 
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *  under the License.
 */
package net.sf.tacos.resolvers;

import org.apache.hivemind.Resource;
import org.apache.hivemind.impl.DefaultClassResolver;
import org.apache.hivemind.impl.LocationImpl;
import org.apache.hivemind.util.ClasspathResource;
import org.apache.tapestry.INamespace;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.resolver.ISpecificationResolverDelegate;
import org.apache.tapestry.services.ClassFinder;
import org.apache.tapestry.spec.IComponentSpecification;
import org.apache.tapestry.spec.ComponentSpecification;

/**
 * Loads tapestry templates from the classpath - near their corresponding java file.
 * 
 * @author Andreas Andreou, amplafi.com
 */
public class ClasspathSpecResolverDelegate implements ISpecificationResolverDelegate {
    private ClassFinder classFinder;
    
    public IComponentSpecification findComponentSpecification(
            IRequestCycle cycle, INamespace namespace, String name) {
        return null;
    }
    
    public IComponentSpecification findPageSpecification(IRequestCycle cycle,
            INamespace namespace, String name) {
        if (namespace.isApplicationNamespace()) {
            String packages = namespace
                    .getPropertyValue("org.apache.tapestry.page-class-packages");
            
            String className = name.replace('/', '.');
            
            Class pageClass = classFinder.findClass(packages, className);
            
            if (pageClass == null) {
                return null;
            }
            String fullPath = pageClass.getName().replace('.', '/');
            ClasspathResource html = new ClasspathResource(new DefaultClassResolver(), fullPath + ".html");
            
            if (html.getResourceURL() != null) {
                return setupImplicitPage(namespace.getSpecificationLocation(),
                        new ClasspathResource(new DefaultClassResolver(), fullPath + ".page"));
            }
        }
        return null;
    }
    
    private IComponentSpecification setupImplicitPage(Resource resource, Resource pageResource) {
        IComponentSpecification specification = new ComponentSpecification();
        specification.setPageSpecification(true);
        specification.setSpecificationLocation(pageResource);
        specification.setLocation(new LocationImpl(resource));
        
        return specification;
    }
    
    public void setClassFinder(ClassFinder classFinder) {
        this.classFinder = classFinder;
    }
    
}

