/*******************************************************************************
 * Copyright (c) 2005 Gabriel Handford.
 * All rights reserved.
 *
 * Contributors:
 *    Gabriel Handford -
 *******************************************************************************/
package net.sf.tacos.services;

import java.util.List;

/**
 * This class describes a tapestry page.
 *
 * @author Gabriel Handford
 */
public interface PageInfo {

    /**
     * Get the page description.
     * @return The page description.
     */
    public String getDesc();

    /**
     * Get the unique page name.
     * @return Page name
     */
    public String getName();

    /**
     * Get the permission associated with page, if any.
     * @return Permission
     */
    public String getPermission();

    /**
     * Get the parent to this page.
     * @return parent The parent.
     */
    public PageInfo getParent();

    /**
     * Set the parent.
     * @param parent The parent.
     */
    public void setParent(PageInfo parent);

    /**
     * Adds the specified child node.
     * @param child Child page
     */
    public void addChild(PageInfo child);
    
    /**
     * Get the children.
     * @return Children
     */
    public List getChildrenNames();

    /**
     * Check if page should be included in navigation.
     * @return Whether page should be included in navigation.
     */
    public boolean isNavigable();
}