/*******************************************************************************
 * Copyright (c) 2005 Gabriel Handford.
 * All rights reserved.
 *
 * Contributors:
 *    Gabriel Handford -
 *******************************************************************************/
package net.sf.tacos.services.impl;

import java.util.List;

import net.sf.tacos.services.CategoryInfo;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;

/**
 * This class describes a site map category containing page names.
 *
 * @author Gabriel Handford
 */
public class CategoryInfoImpl implements CategoryInfo {

    private String name;
    private List pageNames;
    private String image;
    private String inactiveImage;

    /**
     * Construct named category with list of pages.
     * @param name The category name
     * @param pageNames List of the page names in this category
     */
    public CategoryInfoImpl(String name, List pageNames) {
        this.name = name;
        this.pageNames = pageNames;
    }

    /**
     * Construct named category with list of pages, active and inactive image.
     * @param name The category name
     * @param pageNames List of the page names in this category
     */
    public CategoryInfoImpl(String name, List pageNames,
            String image, String inactiveImage) {
        this.name = name;
        this.pageNames = pageNames;
        this.image = image;
        this.inactiveImage = inactiveImage;
    }
    
    /**
     * Get category name.
     * @return name The category name
     */
    public String getName() {
        return name;
    }

    /**
     * Get the default page.
     * @return The default page, or the first page, or null.
     */
    public String getDefaultPage() {
        if (pageNames.size() > 0)
            return (String)pageNames.get(0);
        else
            return null;
    }

    /**
     * Get list of pages in category.
     * @return pageNames The page names in this category
     */
    public List getPageNames() {
        return pageNames;
    }

    /**
     * Returns active image for category.
     * @return
     */
    public String getImage()
    {
        return image;
    }
    
    /**
     * Returns the inactive image for category.
     * @return
     */
    public String getInactiveImage()
    {
        return inactiveImage;
    }
    
    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this, ToStringStyle.DEFAULT_STYLE)
                .append("name", this.name).append("pageNames", this.pageNames)
                .toString();
    }

    /**
     * @see java.lang.Object#equals(Object)
     */
    public boolean equals(Object object) {
        if (object == this) {
            return true;
        }
        if (!(object instanceof CategoryInfo)) {
            return false;
        }
        CategoryInfo rhs = (CategoryInfo) object;
        return new EqualsBuilder().append(this.name, rhs.getName())
                .isEquals();
    }
    
    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return new HashCodeBuilder(1471056309, 288593091).append(this.name)
                .toHashCode();
    }
}
