/*******************************************************************************
 * Copyright (c) 2005 Gabriel Handford.
 * All rights reserved.
 *
 * Contributors:
 *    Gabriel Handford -
 *******************************************************************************/
package net.sf.tacos.services.impl;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import net.sf.tacos.services.PageInfo;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;

/**
 * This class describes a tapestry page.
 *
 * @author Gabriel Handford
 */
public class PageInfoImpl implements PageInfo {

    private String name;
    private String path;
    private String desc;
    private String perm;
    private PageInfo parent;
    private List children;
    private boolean isNavigable;

    /**
     * Create a tapestry page descriptor.
     * @param name The page name.
     * @param desc The page description.
     * @param perm The page permission (access code needed to access the page).
     * @param children List of chile pages.
     * @param isNavigable Whether or not page should be included in the navigation.
     */
    public PageInfoImpl(String name, String desc, 
            String perm, List children, boolean isNavigable) {
        this.name = name;
        this.desc = desc;
        this.perm = perm;
        this.children = children;
        this.isNavigable = isNavigable;
    }

    /**
     * @return The description
     */
    public String getDesc() {
        return desc;
    }

    /**
     * @return The page name
     */
    public String getName() {
        return name;
    }

    /**
     * @return The permission access code
     */
    public String getPermission() {
        return perm;
    }

    /**
     * @return Returns the parent.
     */
    public PageInfo getParent() {
        return parent;
    }

    /**
     * Set the parent.
     * @param parent The parent.
     */
    public void setParent(PageInfo parent) {
        this.parent = parent;
    }

    /**
     * 
     * {@inheritDoc}
     */
    public void addChild(PageInfo child)
    {
        children.add(child);
    }
    
    /**
     * Check whether this page should be included in any page listings.
     * @return True if this page should be included in any page listings, false otherwise
     */
    public boolean isNavigable() { return isNavigable; }
    
    /**
     * Get the children.
     * @return children
     */
    public List getChildrenNames() {
        if (children == null) return Arrays.asList(new Object[0]);
        List list = new ArrayList(children.size());
        for(int i = 0; i < children.size(); i++)
            list.add(((PageInfo)children.get(i)).getName());
        return list;
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this, ToStringStyle.MULTI_LINE_STYLE)
            .append("name", this.name)
            .append("desc", this.desc)
            .append("perm", this.perm)
            .append("parent", (this.parent != null ? parent.getName() : "none"))
            .append("isNavigable", this.isNavigable)
            .toString();
    }
}