// Copyright May 16, 2006 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package net.sf.tacos.util;

import org.apache.tapestry.IMarkupWriter;
import org.apache.tapestry.NestedMarkupWriter;
import org.apache.tapestry.json.JSONObject;
import org.apache.tapestry.markup.Attribute;

/**
 * A {@link IMarkupWriter} that only gathers attributes and
 * attribute values into a JSON structure.
 */
public class JSONMarkupWriter implements IMarkupWriter
{
    private JSONObject json;
    
    public JSONMarkupWriter()
    {
        json = new JSONObject();
    }
    
    public JSONObject getAttributes()
    {
        return json;
    }

    public void attribute(String name, int value)
    {
        json.put(name, value);
    }

    public void attribute(String name, boolean value)
    {
        json.put(name, value);
    }

    public void attribute(String name, String value)
    {
        // look for hints
        if (name.endsWith("AsInt"))
            attribute(name.substring(0, name.length() - 5), Integer.parseInt(value));
        else if (name.endsWith("AsBoolean"))
            attribute(name.substring(0, name.length() - 9), Boolean.valueOf(value).booleanValue());
        
        // look for known names
        else if (name.endsWith("Duration"))
            attribute(name, Integer.parseInt(value));
        else
            json.put(name, value);
    }

    public void attributeRaw(String name, String value)
    {
        json.put(name, value);
    }
    
    public void appendAttribute(String name, int value)
    {        
    }

    public void appendAttribute(String name, boolean value)
    {        
    }

    public void appendAttribute(String name, String value)
    {        
    }

    public void appendAttributeRaw(String name, String value)
    {        
    }
    
    public boolean hasAttribute(String name)
    {
        return false;
    }
    
    public Attribute getAttribute(String name)
    {
        return null;
    }
    
    public Attribute removeAttribute(String name)
    {
        return null;
    }
    
    public void clearAttributes()
    {        
    }

    public void begin(String name)
    {
    }

    public void beginEmpty(String name)
    {
    }

    public boolean checkError()
    {
        return false;
    }

    public void close()
    {
    }

    public void closeTag()
    {
    }

    public void comment(String value)
    {
    }

    public void end()
    {
    }

    public void end(String name)
    {
    }

    public void flush()
    {
    }

    public NestedMarkupWriter getNestedWriter()
    {
        return null;
    }

    public void print(char[] data, int offset, int length)
    {
    }

    public void print(char[] data, int offset, int length, boolean raw)
    {
    }

    public void print(char value)
    {
    }

    public void print(int value)
    {
    }

    public void print(String value)
    {
    }

    public void print(String value, boolean raw)
    {
    }

    public void println()
    {
    }

    public void printRaw(char[] buffer, int offset, int length)
    {
    }

    public void printRaw(String value)
    {
    }

    public String getContentType()
    {
        return "";
    }
    
}
